@@ -1,7 +1,11 @@
 #! /usr/bin/env python
+
+# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
 from saqc.core.modules.breaks import Breaks
-from saqc.core.modules.noise import Noise
 from saqc.core.modules.changepoints import ChangePoints
 from saqc.core.modules.constants import Constants
 from saqc.core.modules.curvefit import Curvefit
@@ -9,10 +13,11 @@ from saqc.core.modules.drift import Drift
 from saqc.core.modules.flagtools import FlagTools
 from saqc.core.modules.generic import Generic
 from saqc.core.modules.interpolation import Interpolation
+from saqc.core.modules.noise import Noise
 from saqc.core.modules.outliers import Outliers
 from saqc.core.modules.pattern import Pattern
 from saqc.core.modules.resampling import Resampling
-from saqc.core.modules.residues import Residues
+from saqc.core.modules.residuals import Residuals
 from saqc.core.modules.rolling import Rolling
 from saqc.core.modules.scores import Scores
 from saqc.core.modules.tools import Tools
@@ -33,7 +38,7 @@ class FunctionsMixin(
     Outliers,
     Pattern,
     Resampling,
-    Residues,
+    Residuals,
     Rolling,
     Scores,
     Tools,
