@@ -1,18 +1,26 @@
 #! /usr/bin/env python
+
+# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
 from __future__ import annotations
 
-from typing import Union, Callable
+from typing import Callable, Union
 
 import numpy as np
 import pandas as pd
 
-from saqc.constants import UNFLAGGED
 import saqc
+import saqc.funcs
+from saqc.constants import UNFLAGGED
 from saqc.funcs.interpolation import _SUPPORTED_METHODS
+from saqc.lib.docurator import doc
 
 
 class Interpolation:
+    @doc(saqc.funcs.interpolation.interpolateByRolling.__doc__)
     def interpolateByRolling(
         self,
         field: str,
@@ -25,6 +33,7 @@ class Interpolation:
     ) -> saqc.SaQC:
         return self._defer("interpolateByRolling", locals())
 
+    @doc(saqc.funcs.interpolation.interpolateInvalid.__doc__)
     def interpolateInvalid(
         self,
         field: str,
@@ -37,6 +46,7 @@ class Interpolation:
     ) -> saqc.SaQC:
         return self._defer("interpolateInvalid", locals())
 
+    @doc(saqc.funcs.interpolation.interpolateIndex.__doc__)
     def interpolateIndex(
         self,
         field: str,
