@@ -1,12 +1,20 @@
 #! /usr/bin/env python
+
+# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
 from __future__ import annotations
 
-from saqc.constants import BAD
 import saqc
+import saqc.funcs
+from saqc.constants import BAD
+from saqc.lib.docurator import doc
 
 
 class Pattern:
+    @doc(saqc.funcs.pattern.flagPatternByDTW.__doc__)
     def flagPatternByDTW(
         self,
         field,
@@ -15,6 +23,6 @@ class Pattern:
         normalize=True,
         plot=False,
         flag=BAD,
-        **kwargs
+        **kwargs,
     ) -> saqc.SaQC:
         return self._defer("flagPatternByDTW", locals())
