@@ -1,30 +1,32 @@
 #! /usr/bin/env python
+
+# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
+
+
 from __future__ import annotations
 
 import functools
-from typing import Optional, Tuple, Sequence, Callable, Optional
-from typing_extensions import Literal
+import inspect
+from typing import Callable, Optional, Sequence, Tuple
 
 import numpy as np
 import pandas as pd
-
-from scipy import stats
 from scipy.optimize import curve_fit
 from scipy.spatial.distance import pdist
+from typing_extensions import Literal
 
 from dios import DictOfSeries
 from saqc.constants import BAD
-from saqc.core.register import register, flagging
-from saqc.core import Flags
+from saqc.core.register import Flags, flagging, register
 from saqc.funcs.changepoints import _assignChangePointCluster
-from saqc.funcs.tools import dropField, copyField
-
-from saqc.lib.tools import detectDeviants, toSequence, filterKwargs
-from saqc.lib.ts_operators import linearDriftModel, expDriftModel
+from saqc.funcs.tools import copyField, dropField
+from saqc.lib.tools import detectDeviants, filterKwargs, toSequence
+from saqc.lib.ts_operators import expDriftModel, linearDriftModel
 from saqc.lib.types import CurveFitter
-from saqc.lib.ts_operators import linearDriftModel, expDriftModel
-
 
 LinkageString = Literal[
     "single", "complete", "average", "weighted", "centroid", "median", "ward"
@@ -33,7 +35,13 @@ LinkageString = Literal[
 MODELDICT = {"linear": linearDriftModel, "exponential": expDriftModel}
 
 
-@flagging()
+@register(
+    mask=["field"],
+    demask=["field"],
+    squeeze=["field"],  # reference is written !
+    multivariate=True,
+    handles_target=False,
+)
 def flagDriftFromNorm(
     data: DictOfSeries,
     field: Sequence[str],
@@ -50,81 +58,81 @@ def flagDriftFromNorm(
     **kwargs,
 ) -> Tuple[DictOfSeries, Flags]:
     """
-    The function flags value courses that significantly deviate from a group of normal value courses.
+    Flags data that deviates from an avarage data course.
 
-    "Normality" is determined in terms of a maximum spreading distance, that members of a normal group must not exceed.
-    In addition, only a group is considered "normal" if it contains more then `frac` percent of the
-    variables in "field".
+    "Normality" is determined in terms of a maximum spreading distance,
+    that members of a normal group must not exceed. In addition, only a group is considered
+    "normal" if it contains more then `frac` percent of the variables in "field".
 
     See the Notes section for a more detailed presentation of the algorithm
 
     Parameters
     ----------
-    data : dios.DictOfSeries
-        A dictionary of pandas.Series, holding all the data.
-    field : list of str
-        List of fieldnames in data, determining which variables are to be included into the flagging process.
-    flags : saqc.Flags
-        A flags object, holding flags and additional informations related to `data`.
+    data : DictOfSeries
+        The data container.
+
+    field : str
+        A column in flags and data.
+
+    flags : Flags
+        The flags container.
+
     freq : str
-        An offset string, determining the size of the seperate datachunks that the algorihm is to be piecewise
-        applied on.
+        Frequency, that split the data in chunks.
+
     spread : float
-        A parameter limiting the maximum "spread" of the timeseries, allowed in the "normal" group. See Notes section
-        for more details.
+        Maximum spread allowed in the group of *normal* data. See Notes section for more details.
+
     frac : float, default 0.5
-        Has to be in [0,1]. Determines the minimum percentage of variables, the "normal" group has to comprise to be the
-        normal group actually. The higher that value, the more stable the algorithm will be with respect to false
-        positives. Also, nobody knows what happens, if this value is below 0.5.
-    metric : Callable[[numpy.array, numpy.array], float]
-        A distance function. It should be a function of 2 1-dimensional arrays and return a float scalar value.
-        This value is interpreted as the distance of the two input arrays. The default is the averaged manhatten metric.
-        See the Notes section to get an idea of why this could be a good choice.
+        Fraction defining the normal group. Use a value from the interval [0,1].
+        The higher the value, the more stable the algorithm will be. For values below
+        0.5 the results are undefined.
+
+    metric : Callable, default ``lambda x,y:pdist(np.array([x,y]),metric="cityblock")/len(x)``
+        Distance function that takes two arrays as input and returns a scalar float.
+        This value is interpreted as the distance of the two input arrays.
+        Defaults to the `averaged manhattan metric` (see Notes).
+
     method : {"single", "complete", "average", "weighted", "centroid", "median", "ward"}, default "single"
-        The linkage method used for hierarchical (agglomerative) clustering of the timeseries.
-        See the Notes section for more details.
-        The keyword gets passed on to scipy.hierarchy.linkage. See its documentation to learn more about the different
-        keywords (References [1]).
-        See wikipedia for an introduction to hierarchical clustering (References [2]).
+        Linkage method used for hierarchical (agglomerative) clustering of the data.
+        `method` is directly passed to ``scipy.hierarchy.linkage``. See its documentation [1] for
+        more details. For a general introduction on hierarchical clustering see [2].
+
     flag : float, default BAD
         flag to set.
 
     Returns
     -------
     data : dios.DictOfSeries
-        A dictionary of pandas.Series, holding all the data.
     flags : saqc.Flags
-        The quality flags of data
-        Flags values may have changed relatively to the input flags.
 
     Notes
     -----
     following steps are performed for every data "segment" of length `freq` in order to find the
     "abnormal" data:
 
-    1. Calculate the distances :math:`d(x_i,x_j)` for all :math:`x_i` in parameter `field`. (with :math:`d`
-       denoting the distance function
-       passed to the parameter `metric`.
-    2. Calculate a dendogram with a hierarchical linkage algorithm, specified by the parameter `method`.
-    3. Flatten the dendogram at the level, the agglomeration costs exceed the value given by the parameter `spread`
-    4. check if there is a cluster containing more than `frac` percentage of the variables in field.
+    1. Calculate distances :math:`d(x_i,x_j)` for all :math:`x_i` in parameter `field`.
+       (with :math:`d` denoting the distance function, specified by `metric`.
+    2. Calculate a dendogram with a hierarchical linkage algorithm, specified by `method`.
+    3. Flatten the dendogram at the level, the agglomeration costs exceed `spread`
+    4. check if a cluster containing more than `frac` variables.
 
         1. if yes: flag all the variables that are not in that cluster (inside the segment)
         2. if no: flag nothing
 
-    The main parameter giving control over the algorithms behavior is the `spread` parameter, that determines
-    the maximum spread of a normal group by limiting the costs, a cluster agglomeration must not exceed in every
-    linkage step.
-    For singleton clusters, that costs just equal half the distance, the timeseries in the clusters, have to
-    each other. So, no timeseries can be clustered together, that are more then
-    2*`spread` distanted from each other.
-    When timeseries get clustered together, this new clusters distance to all the other timeseries/clusters is
-    calculated according to the linkage method specified by `method`. By default, it is the minimum distance,
-    the members of the clusters have to each other.
-    Having that in mind, it is advisable to choose a distance function, that can be well interpreted in the units
-    dimension of the measurement and where the interpretation is invariant over the length of the timeseries.
-    That is, why, the "averaged manhatten metric" is set as the metric default, since it corresponds to the
-    averaged value distance, two timeseries have (as opposed by euclidean, for example).
+    The main parameter giving control over the algorithms behavior is the `spread` parameter,
+    that determines the maximum spread of a normal group by limiting the costs, a cluster
+    agglomeration must not exceed in every linkage step.
+    For singleton clusters, that costs just equal half the distance, the data in the
+    clusters, have to each other. So, no data can be clustered together, that are more then
+    2*`spread` distances away from each other. When data get clustered together, this new
+    clusters distance to all the other data/clusters is calculated according to the linkage
+    method specified by `method`. By default, it is the minimum distance, the members of the
+    clusters have to each other. Having that in mind, it is advisable to choose a distance
+    function, that can be well interpreted in the units dimension of the measurement and where
+    the interpretation is invariant over the length of the data. That is, why,
+    the "averaged manhattan metric" is set as the metric default, since it corresponds to the
+    averaged value distance, two data sets have (as opposed by euclidean, for example).
 
     References
     ----------
@@ -134,6 +142,7 @@ def flagDriftFromNorm(
         [2] https://en.wikipedia.org/wiki/Hierarchical_clustering
     """
     fields = toSequence(field)
+
     data_to_flag = data[fields].to_df()
     data_to_flag.dropna(inplace=True)
 
@@ -151,13 +160,12 @@ def flagDriftFromNorm(
     return data, flags
 
 
-# default multivariate flagging
 @register(
     mask=["field", "reference"],
     demask=["field", "reference"],
     squeeze=["field", "reference"],  # reference is written !
     multivariate=True,
-    handles_target=True,
+    handles_target=False,
 )
 def flagDriftFromReference(
     data: DictOfSeries,
@@ -170,64 +178,66 @@ def flagDriftFromReference(
         np.array([x, y]), metric="cityblock"
     )
     / len(x),
-    target=None,
     flag: float = BAD,
     **kwargs,
 ) -> Tuple[DictOfSeries, Flags]:
     """
-    The function flags value courses that deviate from a reference course by a margin exceeding a certain threshold.
+    Flags data that deviates from a reference course.
 
-    The deviation is measured by the distance function passed to parameter metric.
+    The deviation is measured by a passed distance function.
 
     Parameters
     ----------
-    data : dios.DictOfSeries
-        A dictionary of pandas.Series, holding all the data.
-    field : list of str
-        List of fieldnames in data, determining wich variables are to be included into the flagging process.
-    flags : saqc.Flags
-        A flags object, holding flags and additional informations related to `data`.
-    reference : str
-        The reference variable, the deviation from wich determines the flagging.
+    data : DictOfSeries
+        The data container.
+
+    field : str
+        A column in flags and data.
+
+    flags : Flags
+        The flags container.
+
     freq : str
-        An offset string, determining the size of the seperate datachunks that the algorihm is to be piecewise
-        applied on.
+        Frequency, that split the data in chunks.
+
+    reference : str
+        Reference variable, the deviation is calculated from.
+
     thresh : float
-        The threshod by wich normal variables can deviate from the reference variable at max.
-    metric : Callable[(numpyp.array, numpy-array), float]
-        A distance function. It should be a function of 2 1-dimensional arrays and return a float scalar value.
-        This value is interpreted as the distance of the two input arrays. The default is the averaged manhatten metric.
-        See the Notes section to get an idea of why this could be a good choice.
+        Maximum deviation from reference.
+
+    metric : Callable
+        Distance function. Takes two arrays as input and returns a scalar float.
+        This value is interpreted as the mutual distance of the two input arrays.
+        Defaults to the `averaged manhattan metric` (see Notes).
+
     target : None
-        ignored.
+        Ignored.
+
     flag : float, default BAD
-        flag to set.
+        Flag to set.
 
     Returns
     -------
     data : dios.DictOfSeries
-        A dictionary of pandas.Series, holding all the data.
     flags : saqc.Flags
-        The quality flags of data
-        Flags values may have changed relatively to the input flags.
 
     Notes
     -----
-    it is advisable to choose a distance function, that can be well interpreted in the units
-    dimension of the measurement and where the interpretation is invariant over the length of the timeseries.
-    That is, why, the "averaged manhatten metric" is set as the metric default, since it corresponds to the
-    averaged value distance, two timeseries have (as opposed by euclidean, for example).
+    It is advisable to choose a distance function, that can be well interpreted in
+    the units dimension of the measurement and where the interpretation is invariant over the
+    length of the data. That is, why, the "averaged manhatten metric" is set as the metric
+    default, since it corresponds to the averaged value distance, two data sets have (as opposed
+    by euclidean, for example).
     """
-    if target:
-        raise NotImplementedError("target is not implemented for this function")
 
     fields = toSequence(field)
-    data_to_flag = data[fields].to_df().dropna()
 
-    fields = list(fields)
     if reference not in fields:
         fields.append(reference)
 
+    data_to_flag = data[fields].to_df().dropna()
+
     segments = data_to_flag.groupby(pd.Grouper(freq=freq))
     for segment in segments:
 
@@ -245,156 +255,6 @@ def flagDriftFromReference(
     return data, flags
 
 
-# default multivariate flagging
-@register(
-    mask=["set_1", "set_2"],
-    demask=["set_1", "set_2"],
-    squeeze=["set_1", "set_2"],
-    multivariate=True,
-    handles_target=True,
-)
-def flagDriftFromScaledNorm(
-    data: DictOfSeries,
-    field: str,
-    flags: Flags,
-    set_1: Sequence[str],
-    set_2: Sequence[str],
-    freq: str,
-    spread: float,
-    frac: float = 0.5,
-    metric: Callable[[np.ndarray, np.ndarray], float] = lambda x, y: pdist(
-        np.array([x, y]), metric="cityblock"
-    )
-    / len(x),
-    method: LinkageString = "single",
-    target: str = None,
-    flag: float = BAD,
-    **kwargs,
-) -> Tuple[DictOfSeries, Flags]:
-    """
-    The function linearly rescales one set of variables to another set of variables
-    with a different scale and then flags value courses that significantly deviate
-    from a group of normal value courses.
-
-    The two sets of variables can be linearly scaled one to another and hence the
-    scaling transformation is performed via linear regression: A linear regression is
-    performed on each pair of variables giving a slope and an intercept. The
-    transformation is then calculated a the median of all the calculated slopes and
-    intercepts.
-
-    Once the transformation is performed, the function flags those values,
-    that deviate from a group of normal values. "Normality" is determined in terms of
-    a maximum spreading distance, that members of a normal group must not exceed. In
-    addition, only a group is considered "normal" if it contains more then `frac`
-    percent of the variables in "fields".
-
-    Parameters
-    ----------
-    data : dios.DictOfSeries
-        A dictionary of pandas.Series, holding all the data.
-
-    field : str
-        A dummy parameter.
-
-    flags : saqc.Flags
-        A flags object, holding flags and additional informations related to `data`.
-
-    set_1 : str
-        The first set of fieldnames in data to be included into the flagging process.
-
-    set_2 : str
-        The second set of fieldnames in data to be included into the flagging process.
-
-    freq : str
-        An offset string, determining the size of the seperate datachunks that the
-        algorihm is to be piecewise applied on.
-
-    spread : float
-        A parameter limiting the maximum "spread" of the timeseries, allowed in the
-        "normal" group. See Notes section for more details.
-
-    frac : float, default 0.5
-        Has to be in [0,1]. Determines the minimum percentage of variables,
-        the "normal" group has to comprise to be the normal group actually. The
-        higher that value, the more stable the algorithm will be with respect to
-        false positives. Also, nobody knows what happens, if this value is below 0.5.
-
-    metric : Callable[(numpyp.array, numpy-array), float]
-        A distance function. It should be a function of 2 1-dimensional arrays and
-        return a float scalar value. This value is interpreted as the distance of the
-        two input arrays. The default is the averaged manhatten metric. See the Notes
-        section to get an idea of why this could be a good choice.
-
-    method : str, default "single"
-        The linkage method used for hierarchical (agglomerative) clustering of the
-        timeseries. See the Notes section for more details. The keyword gets passed
-        on to scipy.hierarchy.linkage. See its documentation to learn more about the
-        different keywords (References [1]). See wikipedia for an introduction to
-        hierarchical clustering (References [2]).
-
-    target : None
-        ignored
-
-    flag : float, default BAD
-        flag to set.
-
-    Returns
-    -------
-    data : dios.DictOfSeries
-        A dictionary of pandas.Series, holding all the data.
-    flags : saqc.Flags
-        The quality flags of data
-        Flags values may have changed relatively to the input flags.
-
-    References
-    ----------
-    Documentation of the underlying hierarchical clustering algorithm:
-        [1] https://docs.scipy.org/doc/scipy/reference/generated/scipy.cluster.hierarchy.linkage.html
-    Introduction to Hierarchical clustering:
-        [2] https://en.wikipedia.org/wiki/Hierarchical_clustering
-    """
-    if target:
-        raise NotImplementedError("target is not implemented for this function")
-
-    fields = list(set_1) + list(set_2)
-    data_to_flag = data[fields].to_df().dropna()
-
-    convert_slope = []
-    convert_intercept = []
-
-    for field1 in set_1:
-        for field2 in set_2:
-            slope, intercept, *_ = stats.linregress(
-                data_to_flag[field1], data_to_flag[field2]
-            )
-            convert_slope.append(slope)
-            convert_intercept.append(intercept)
-
-    factor_slope = np.median(convert_slope)
-    factor_intercept = np.median(convert_intercept)
-
-    dat = DictOfSeries()
-    for field1 in set_1:
-        dat[field1] = factor_intercept + factor_slope * data_to_flag[field1]
-    for field2 in set_2:
-        dat[field2] = data_to_flag[field2]
-
-    dat_to_flag = dat[fields].to_df()
-
-    segments = dat_to_flag.groupby(pd.Grouper(freq=freq))
-    for segment in segments:
-
-        if segment[1].shape[0] <= 1:
-            continue
-
-        drifters = detectDeviants(segment[1], metric, spread, frac, method, "variables")
-
-        for var in drifters:
-            flags[segment[1].index, fields[var]] = flag
-
-    return data, flags
-
-
 @register(mask=["field"], demask=[], squeeze=[])
 def correctDrift(
     data: DictOfSeries,
@@ -412,17 +272,21 @@ def correctDrift(
 
     Parameters
     ----------
-    data : dios.DictOfSeries
-        A dictionary of pandas.Series, holding all the data.
+    data : DictOfSeries
+        The data container.
+
     field : str
-        The fieldname of the data column, you want to correct.
+        Column in data and flags.
+
     flags : saqc.Flags
-        Container to store quality flags to data.
+        Flags container.
+
     maintenance_field : str
-        The fieldname of the datacolumn holding the support-points information.
-        The maint data is to expected to have following form:
-        The series' timestamp itself represents the beginning of a
-        maintenance event, wheras the values represent the endings of the maintenance intervals.
+        Column holding the support-points information.
+        The data is expected to have the following form:
+        The index of the series represents the beginning of a maintenance
+        event, wheras the values represent its endings.
+
     model : Callable or {'exponential', 'linear'}
         A modelfunction describing the drift behavior, that is to be corrected.
         Either use built-in exponential or linear drift model by passing a string, or pass a custom callable.
@@ -431,9 +295,10 @@ def correctDrift(
         After the data parameter, there can occure an arbitrary number of model calibration arguments in
         the signature.
         See the Notes section for an extensive description.
+
     cal_range : int, default 5
-        The number of values the mean is computed over, for obtaining the value level directly after and
-        directly before maintenance event. This values are needed for shift calibration. (see above description)
+        Number of values to calculate the mean of, for obtaining the value level directly
+        after and directly before a maintenance event. Needed for shift calibration.
 
     Returns
     -------
@@ -445,25 +310,27 @@ def correctDrift(
 
     Notes
     -----
-    It is assumed, that between support points, there is a drift effect shifting the meassurements in a way, that
-    can be described, by a model function ``M(t, *p, origin, target)``. (With ``0<=t<=1``, ``p`` being a parameter
-    set, and ``origin``, ``target`` being floats).
+    It is assumed, that between support points, there is a drift effect shifting the
+    meassurements in a way, that can be described, by a model function M(t, p, origin, target).
+    (With 0<=t<=1, p being a parameter set, and origin, target being floats).
 
     Note, that its possible for the model to have no free parameters p at all. (linear drift mainly)
 
-    The drift model, directly after the last support point (``t=0``),
-    should evaluate to the origin - calibration level (``origin``), and directly before the next support point
-    (``t=1``), it should evaluate to the target calibration level (``target``).::
+    The drift model, directly after the last support point (t=0),
+    should evaluate to the origin - calibration level (origin), and directly before the next
+    support point (t=1), it should evaluate to the target calibration level (target).
 
-        M(0, *p, origin, target) = origin
-        M(1, *p, origin, target) = target
 
-    The model is than fitted to any data chunk in between support points, by optimizing the parameters ``p*``, and
-    thus, obtaining optimal parameterset ``P*``.
+        M(0, p, origin, target) = origin
+        M(1, p, origin, target) = target
+
+
+    The model is than fitted to any data chunk in between support points, by optimizing
+    the parameters p, and thus, obtaining optimal parameterset P.
 
     The new values at t are computed via:::
 
-        new_vals(t) = old_vals(t) + M(t, *P, origin, target) - M_drift(t, *P, origin, new_target)
+        new_vals(t) = old_vals(t) + M(t, P, origin, target) - M_drift(t, P, origin, new_target)
 
     Wheras ``new_target`` represents the value level immediately after the next support point.
 
@@ -473,12 +340,13 @@ def correctDrift(
 
     Linear drift modell (no free parameters).
 
-    >>> M = lambda t, origin, target: origin + t * target
+
+    >>> Model = lambda t, origin, target: origin + t*target
 
     exponential drift model (exponential raise!)
 
     >>> expFunc = lambda t, a, b, c: a + b * (np.exp(c * x) - 1)
-    >>> M = lambda t, p, origin, target: expFunc(t, (target - origin) / (np.exp(abs(c)) - 1), abs(c))
+    >>> Model = lambda t, p, origin, target: expFunc(t, (target - origin) / (np.exp(abs(c)) - 1), abs(c))
 
     Exponential and linear driftmodels are part of the ``ts_operators`` library, under the names
     ``expDriftModel`` and ``linearDriftModel``.
@@ -722,26 +590,27 @@ def correctOffset(
 def _driftFit(x, shift_target, cal_mean, driftModel):
     x_index = x.index - x.index[0]
     x_data = x_index.total_seconds().values
-    x_data = x_data / x_data[-1]
+    x_data = x_data / x_data[-1] if len(x_data) > 1 else x_data
     y_data = x.values
     origin_mean = np.mean(y_data[:cal_mean])
     target_mean = np.mean(y_data[-cal_mean:])
 
     dataFitFunc = functools.partial(driftModel, origin=origin_mean, target=target_mean)
     # if drift model has free parameters:
-    try:
-        # try fitting free parameters
-        fit_paras, *_ = curve_fit(dataFitFunc, x_data, y_data)
-        data_fit = dataFitFunc(x_data, *fit_paras)
-        data_shift = driftModel(
-            x_data, *fit_paras, origin=origin_mean, target=shift_target
-        )
-    except RuntimeError:
-        # if fit fails -> make no correction
-        data_fit = np.array([0] * len(x_data))
-        data_shift = np.array([0] * len(x_data))
+    if len(inspect.getfullargspec(dataFitFunc).args) > 1:
+        try:
+            # try fitting free parameters
+            fit_paras, *_ = curve_fit(dataFitFunc, x_data, y_data)
+            data_fit = dataFitFunc(x_data, *fit_paras)
+            data_shift = driftModel(
+                x_data, *fit_paras, origin=origin_mean, target=shift_target
+            )
+        except RuntimeError:
+            # if fit fails -> make no correction
+            data_fit = np.array([0] * len(x_data))
+            data_shift = np.array([0] * len(x_data))
     # when there are no free parameters in the model:
-    except ValueError:
+    else:
         data_fit = dataFitFunc(x_data)
         data_shift = driftModel(x_data, origin=origin_mean, target=shift_target)
 
@@ -837,7 +706,7 @@ def assignRegimeAnomaly(
     cluster_field: str,
     spread: float,
     method: LinkageString = "single",
-    metric: Callable[[np.array, np.array], float] = lambda x, y: np.abs(
+    metric: Callable[[np.ndarray, np.ndarray], float] = lambda x, y: np.abs(
         np.nanmean(x) - np.nanmean(y)
     ),
     frac: float = 0.5,
