@@ -1,4 +1,9 @@
 #! /usr/bin/env python
+
+# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
 __all__ = [
     "T",
@@ -10,10 +15,12 @@ __all__ = [
     "OptionalNone",
 ]
 
-from typing import Any, TypeVar, Union, Dict
-from typing_extensions import Protocol, Literal
+from typing import Any, Dict, TypeVar, Union
+
 import numpy as np
 import pandas as pd
+from typing_extensions import Literal, Protocol
+
 from dios import DictOfSeries
 
 T = TypeVar("T")
@@ -27,7 +34,7 @@ ExternalFlag = Union[str, float, int]
 # needed for deeper type hinting magic
 class CurveFitter(Protocol):
     def __call__(self, data: np.ndarray, *params: float) -> np.ndarray:
-        ...
+        ...  # pragma: no cover
 
 
 class GenericFunction(Protocol):
@@ -36,7 +43,7 @@ class GenericFunction(Protocol):
     __globals__: Dict[str, Any]
 
     def __call__(self, *args: pd.Series) -> PandasLike:
-        ...
+        ...  # pragma: no cover
 
 
 class OptionalNone:
