@@ -1,13 +1,17 @@
 #! /usr/bin/env python
+
+# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 # -*- coding: utf-8 -*-
 
-import pytest
 import numpy as np
+import pytest
 
-from saqc.constants import *
-from saqc.funcs.constants import flagConstants, flagByVariance
-from saqc.core import initFlagsLike, Flags
-
+from saqc.constants import BAD, UNFLAGGED
+from saqc.core import initFlagsLike
+from saqc.funcs.constants import flagByVariance, flagConstants
 from tests.common import initData
 
 
@@ -33,10 +37,11 @@ def test_constants_flagBasic(data):
 
 
 def test_constants_flagVarianceBased(data):
-    expected = np.arange(5, 25)
     field, *_ = data.columns
     flags = initFlagsLike(data)
-    data, flags_result1 = flagByVariance(data, field, flags, window="1h", flag=BAD)
+    data, flags_result1 = flagByVariance(
+        data, field, flags, window="1h", thresh=0.0005, flag=BAD
+    )
 
     flagscol = flags_result1[field]
     assert np.all(flagscol[5:25] == BAD)
