@@ -1,12 +1,17 @@
+# SPDX-FileCopyrightText: 2021 Helmholtz-Zentrum für Umweltforschung GmbH - UFZ
+#
+# SPDX-License-Identifier: GPL-3.0-or-later
+
+import numpy as np
+import pandas as pd
 import pytest
 
-from saqc.lib.plotting import makeFig
-import pandas as pd
-import numpy as np
-import saqc
 import dios
+import saqc
+from saqc.lib.plotting import makeFig
 
 
+@pytest.mark.slow
 def test_makeFig():
     # just testing for no errors to occure...
     data = dios.DictOfSeries(
@@ -30,5 +35,5 @@ def test_makeFig():
     d_saqc = d_saqc.plot(field="data", path=dummy_path, history="valid", stats=True)
     d_saqc = d_saqc.plot(field="data", path=dummy_path, history="complete")
     d_saqc = d_saqc.plot(
-        field="data", path=dummy_path, ax_kwargs={"ylim": "2000-10"}, stats=True
+        field="data", path=dummy_path, ax_kwargs={"ylabel": "data is data"}, stats=True
     )
